@@ -234,7 +234,7 @@ class AnchoredAnnotation(object):
             data = pd.DataFrame({index_column: data})
 
         self._name = name
-        self._data = data.reset_index()
+        self._data = data.reset_index(drop=True)
         self._original_columns = data.columns
         self._max_distance = max_distance
 
@@ -257,7 +257,7 @@ class AnchoredAnnotation(object):
                 self._data[self._index_column_base] = mesh.map_indices_to_unmasked(
                     data[index_column]
                 )
-                self._data[self._index_column_filt] = data[index_column]
+                self._data[self._index_column_filt] = data[index_column].values
 
         if self._defined_index:
             self._orig_col_plus_index = list(self._original_columns)
@@ -309,6 +309,10 @@ class AnchoredAnnotation(object):
     def index_column(self):
         return self._index_column_filt
 
+    @property
+    def index_column_original(self):
+        return self._index_column_base
+
     @property
     def _is_valid(self):
         return self._data[self._valid_column]
@@ -810,6 +814,7 @@ class Meshwork(object):
                 shape_function=shape_function,
                 collapse_function=collapse_function,
                 collapse_params=collapse_params,
+                meta={"root_id": self.seg_id},
             )
             self._reset_indices()
         else:
@@ -1457,8 +1462,9 @@ class Meshwork(object):
     ##########
     # Saving #
     ##########
-
-    def save_meshwork(self, filename, overwrite=False):
+    def save_meshwork(
+        self, filename, overwrite=False, version=meshwork_io.LATEST_VERSION
+    ):
         """Save meshwork to hdf5 file.
 
         Parameters
@@ -1468,7 +1474,7 @@ class Meshwork(object):
         overwrite : bool, optional
             If True, overwrites an existing file. Default is False.
         """
-        meshwork_io._save_meshwork(filename, self, overwrite=overwrite)
+        meshwork_io._save_meshwork(filename, self, overwrite=overwrite, version=version)
 
 
 def load_meshwork(filename):
